#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import pysam

# ----------------------------------------------------------------------------

parser = argparse.ArgumentParser()

parser.add_argument('-b', '--bed', required=True)
parser.add_argument('-c', '--containment', required=True)
parser.add_argument('-r', '--ref-fasta', required=True)

args = parser.parse_args()

# ----------------------------------------------------------------------------

fasta = pysam.FastaFile(args.ref_fasta)
ref_names = [name.split('_')[0] for name in fasta.references]
ref_map = {name : i for i,name in sorted(enumerate(ref_names),
                               key=lambda x: fasta.lengths[x[0]],
                               reverse=True) }

intervals = dict()
with open(args.bed) as f:
  for line in f:
    fields = line.strip().split()
    ctg = fields[3]
    if ctg not in intervals: intervals[ctg] = list()
    # ivl_str = '%s-%s-%s' % (fields[0], fields[1], fields[2])
    ivl = fields[:3]
    intervals[ctg].append((fields[0], int(fields[1]), int(fields[2])))

with open(args.containment, 'w') as out:
  for ctg, ivls in intervals.iteritems():
    # print ivls
    ivls = [i for i in ivls if i[2]-i[1] > 1000]
    sorted_ivls = sorted(ivls, key=lambda x: x[2]-x[1], reverse=True)
    for ivl in sorted_ivls[:3]:
      out.write('R\t%s\t%d\t%d\t%d\n' % (ctg, ref_map[ivl[0]], ivl[1], ivl[2]))
