from flask import Flask

class HTTPMethodOverrideMiddleWare(object):
    allowed_methods = frozenset([
                    "GET",
                    "HEAD",
                    "POST",
                    "DELETE",
                    "PUT",
                    "PATCH",
                    "OPTIONS",
        ])
    bodyless_methods = frozenset([
                    "GET",
                    "HEAD",
                    "OPTIONS", 
                    "DELETE",
        ])

    def __init__(self, app):
        self.app = app

    def __call__(self, environ, start_response):
        method = environ.get("HTTP_X_HTTP_METHOD_OVERRIDE", "").upper()
        if method in self.allowed_methods:
            method = method.encode("ascii", "replace")
            environ["REQUEST_METHOD"] = method
        if method in self.bodyless_methods:
            environ["CONTENT_LENGTH"] = "0"
        return self.app(environ, start_response)

app = Flask(__nam__)
app.wsgi_app = HTTPMethodOverrideMiddleWare(app.wsgi_app)