from magic import multiverse
from magic.card_description import CardDescription
from magic.database import db


def test_base_query_legalities() -> None:
    sql = multiverse.base_query("f.name = 'Mother of Runes'")
    db().execute('SET group_concat_max_len=100000')
    rs = db().select(sql)
    assert len(rs) == 1
    legalities = rs[0]['legalities']
    assert 'Penny Dreadful EMN:Legal' in legalities
    assert 'Penny Dreadful AKH:Legal' not in legalities

def test_supertypes() -> None:
    assert multiverse.supertypes('Legendary Enchantment Creature - God') == ['Legendary']
    assert multiverse.supertypes('Artifact Creature - Construct') == []
    assert multiverse.supertypes('Snow Basic Land - Island') == ['Basic', 'Snow']
    assert multiverse.supertypes('Enchantment') == []
    assert multiverse.supertypes('Creature - Elder Dragon') == []

def test_subtypes() -> None:
    assert multiverse.subtypes('Legendary Enchantment Creature - God') == ['God']
    assert multiverse.subtypes('Artifact Creature - Construct') == ['Construct']
    assert multiverse.subtypes('Snow Basic Land - Island') == ['Island']
    assert multiverse.subtypes('Enchantment') == []
    assert multiverse.subtypes('Creature - Elder Dragon') == ['Elder', 'Dragon']

def test_name_from_card_description() -> None:
    fire_ice: CardDescription = {'object': 'card', 'id': '3f822331-315e-4297-bb69-f1069032c6c5', 'oracle_id': 'ae92942b-919c-4ea9-b693-85fcef765d5a', 'multiverse_ids': [456821], 'mtgo_id': 70527, 'tcgplayer_id': 180771, 'name': 'Fire // Ice', 'lang': 'en', 'released_at': '2018-12-07', 'uri': 'https://api.scryfall.com/cards/3f822331-315e-4297-bb69-f1069032c6c5', 'scryfall_uri': 'https://scryfall.com/card/uma/225/fire-ice?utm_source=api', 'layout': 'split', 'highres_image': True, 'image_uris': {'small': 'https://img.scryfall.com/cards/small/front/3/f/3f822331-315e-4297-bb69-f1069032c6c5.jpg?1547518354', 'normal': 'https://img.scryfall.com/cards/normal/front/3/f/3f822331-315e-4297-bb69-f1069032c6c5.jpg?1547518354', 'large': 'https://img.scryfall.com/cards/large/front/3/f/3f822331-315e-4297-bb69-f1069032c6c5.jpg?1547518354', 'png': 'https://img.scryfall.com/cards/png/front/3/f/3f822331-315e-4297-bb69-f1069032c6c5.png?1547518354', 'art_crop': 'https://img.scryfall.com/cards/art_crop/front/3/f/3f822331-315e-4297-bb69-f1069032c6c5.jpg?1547518354', 'border_crop': 'https://img.scryfall.com/cards/border_crop/front/3/f/3f822331-315e-4297-bb69-f1069032c6c5.jpg?1547518354'}, 'mana_cost': '{1}{R} // {1}{U}', 'cmc': 4.0, 'type_line': 'Instant // Instant', 'colors': ['R', 'U'], 'color_identity': ['R', 'U'], 'card_faces': [{'object': 'card_face', 'name': 'Fire', 'mana_cost': '{1}{R}', 'type_line': 'Instant', 'oracle_text': 'Fire deals 2 damage divided as you choose among one or two targets.', 'artist': 'Dan Scott', 'artist_id': 'f852fa13-137e-40f2-bbc1-0f01df09c0e0', 'illustration_id': '511174a2-18dc-4582-9de1-c04871eb77de'}, {'object': 'card_face', 'name': 'Ice', 'mana_cost': '{1}{U}', 'type_line': 'Instant', 'oracle_text': 'Tap target permanent.\nDraw a card.', 'artist': 'Dan Scott', 'artist_id': 'f852fa13-137e-40f2-bbc1-0f01df09c0e0'}], 'legalities': {'standard': 'not_legal', 'future': 'not_legal', 'historic': 'not_legal', 'pioneer': 'not_legal', 'modern': 'not_legal', 'legacy': 'legal', 'pauper': 'legal', 'vintage': 'legal', 'penny': 'not_legal', 'commander': 'legal', 'brawl': 'not_legal', 'duel': 'legal', 'oldschool': 'not_legal'}, 'games': ['mtgo', 'paper'], 'reserved': False, 'foil': True, 'nonfoil': True, 'oversized': False, 'promo': False, 'reprint': True, 'variation': False, 'set': 'uma', 'set_name': 'Ultimate Masters', 'set_type': 'masters', 'set_uri': 'https://api.scryfall.com/sets/2ec77b94-6d47-4891-a480-5d0b4e5c9372', 'set_search_uri': 'https://api.scryfall.com/cards/search?order=set&q=e%3Auma&unique=prints', 'scryfall_set_uri': 'https://scryfall.com/sets/uma?utm_source=api', 'rulings_uri': 'https://api.scryfall.com/cards/3f822331-315e-4297-bb69-f1069032c6c5/rulings', 'prints_search_uri': 'https://api.scryfall.com/cards/search?order=released&q=oracleid%3Aae92942b-919c-4ea9-b693-85fcef765d5a&unique=prints', 'collector_number': '225', 'digital': False, 'rarity': 'common', 'card_back_id': '0aeebaf5-8c7d-4636-9e82-8c27447861f7', 'artist': 'Dan Scott', 'artist_ids': ['f852fa13-137e-40f2-bbc1-0f01df09c0e0'], 'illustration_id': '511174a2-18dc-4582-9de1-c04871eb77de', 'border_color': 'black', 'frame': '2015', 'full_art': False, 'textless': False, 'booster': True, 'story_spotlight': False, 'edhrec_rank': 17833, 'related_uris': {'gatherer': 'https://gatherer.wizards.com/Pages/Card/Details.aspx?multiverseid=456821', 'tcgplayer_decks': 'https://decks.tcgplayer.com/magic/deck/search?contains=Fire+%2F%2F+Ice&page=1&partner=Scryfall&utm_campaign=affiliate&utm_medium=scryfall&utm_source=scryfall', 'edhrec': 'https://edhrec.com/route/?cc=Fire+%2F%2F+Ice', 'mtgtop8': 'https://mtgtop8.com/search?MD_check=1&SB_check=1&cards=Fire+%2F%2F+Ice'}}
    hanweir_garrison: CardDescription = {'object': 'card', 'id': '6c81b141-4c00-4779-b7d4-d0518f733682', 'oracle_id': '7cb29569-48e1-4782-9906-fad155ebfafe', 'multiverse_ids': [], 'tcgplayer_id': 120737, 'name': 'Hanweir Garrison', 'lang': 'en', 'released_at': '2016-07-22', 'uri': 'https://api.scryfall.com/cards/6c81b141-4c00-4779-b7d4-d0518f733682', 'scryfall_uri': 'https://scryfall.com/card/pemn/130as/hanweir-garrison?utm_source=api', 'layout': 'meld', 'highres_image': True, 'image_uris': {'small': 'https://img.scryfall.com/cards/small/front/6/c/6c81b141-4c00-4779-b7d4-d0518f733682.jpg?1562917746', 'normal': 'https://img.scryfall.com/cards/normal/front/6/c/6c81b141-4c00-4779-b7d4-d0518f733682.jpg?1562917746', 'large': 'https://img.scryfall.com/cards/large/front/6/c/6c81b141-4c00-4779-b7d4-d0518f733682.jpg?1562917746', 'png': 'https://img.scryfall.com/cards/png/front/6/c/6c81b141-4c00-4779-b7d4-d0518f733682.png?1562917746', 'art_crop': 'https://img.scryfall.com/cards/art_crop/front/6/c/6c81b141-4c00-4779-b7d4-d0518f733682.jpg?1562917746', 'border_crop': 'https://img.scryfall.com/cards/border_crop/front/6/c/6c81b141-4c00-4779-b7d4-d0518f733682.jpg?1562917746'}, 'mana_cost': '{2}{R}', 'cmc': 3.0, 'type_line': 'Creature — Human Soldier', 'oracle_text': 'Whenever Hanweir Garrison attacks, create two 1/1 red Human creature tokens that are tapped and attacking.\n(Melds with Hanweir Battlements.)', 'power': '2', 'toughness': '3', 'colors': ['R'], 'color_identity': ['R'], 'all_parts': [{'object': 'related_card', 'id': '6c81b141-4c00-4779-b7d4-d0518f733682', 'component': 'meld_part', 'name': 'Hanweir Garrison', 'type_line': 'Creature — Human Soldier', 'uri': 'https://api.scryfall.com/cards/6c81b141-4c00-4779-b7d4-d0518f733682'}, {'object': 'related_card', 'id': 'e6a5fea5-6a36-48e2-9602-e0742ad6dbc4', 'component': 'meld_result', 'name': 'Hanweir, the Writhing Township', 'type_line': 'Legendary Creature — Eldrazi Ooze', 'uri': 'https://api.scryfall.com/cards/e6a5fea5-6a36-48e2-9602-e0742ad6dbc4'}, {'object': 'related_card', 'id': 'a32261f2-164f-4433-bc4c-b5ac591e6a59', 'component': 'meld_part', 'name': 'Hanweir Battlements', 'type_line': 'Land', 'uri': 'https://api.scryfall.com/cards/a32261f2-164f-4433-bc4c-b5ac591e6a59'}], 'legalities': {'standard': 'not_legal', 'future': 'not_legal', 'historic': 'not_legal', 'pioneer': 'legal', 'modern': 'legal', 'legacy': 'legal', 'pauper': 'not_legal', 'vintage': 'legal', 'penny': 'legal', 'commander': 'legal', 'brawl': 'not_legal', 'duel': 'legal', 'oldschool': 'not_legal'}, 'games': ['paper'], 'reserved': False, 'foil': True, 'nonfoil': False, 'oversized': False, 'promo': True, 'reprint': True, 'variation': False, 'set': 'pemn', 'set_name': 'Eldritch Moon Promos', 'set_type': 'promo', 'set_uri': 'https://api.scryfall.com/sets/a173a21c-7d0f-4bdd-97df-72ce0b661d62', 'set_search_uri': 'https://api.scryfall.com/cards/search?order=set&q=e%3Apemn&unique=prints', 'scryfall_set_uri': 'https://scryfall.com/sets/pemn?utm_source=api', 'rulings_uri': 'https://api.scryfall.com/cards/6c81b141-4c00-4779-b7d4-d0518f733682/rulings', 'prints_search_uri': 'https://api.scryfall.com/cards/search?order=released&q=oracleid%3A7cb29569-48e1-4782-9906-fad155ebfafe&unique=prints', 'collector_number': '130as', 'digital': False, 'rarity': 'rare', 'flavor_text': '"We\'re ready for anything!"', 'card_back_id': '8e3d4e1e-82e6-4a0a-9b18-5a96683d24c2', 'artist': 'Vincent Proce', 'artist_ids': ['d82b1138-76d3-49f7-9d8c-bc2e2d3e0b0a'], 'illustration_id': '2b2d858a-8c54-413a-a107-cfc587540ac9', 'border_color': 'black', 'frame': '2015', 'full_art': False, 'textless': False, 'booster': False, 'story_spotlight': False, 'promo_types': ['prerelease', 'datestamped', 'setpromo'], 'edhrec_rank': 1112, 'related_uris': {'tcgplayer_decks': 'https://decks.tcgplayer.com/magic/deck/search?contains=Hanweir+Garrison&page=1&partner=Scryfall&utm_campaign=affiliate&utm_medium=scryfall&utm_source=scryfall', 'edhrec': 'https://edhrec.com/route/?cc=Hanweir+Garrison', 'mtgtop8': 'https://mtgtop8.com/search?MD_check=1&SB_check=1&cards=Hanweir+Garrison'}}
    giant_killer: CardDescription = {'object': 'card', 'id': 'f999067b-f7af-4482-9549-108d21be7acc', 'oracle_id': '91e71279-c9d2-4873-916a-59da03a65741', 'multiverse_ids': [], 'name': 'Giant Killer // Chop Down', 'lang': 'en', 'released_at': '2019-10-04', 'uri': 'https://api.scryfall.com/cards/f999067b-f7af-4482-9549-108d21be7acc', 'scryfall_uri': 'https://scryfall.com/card/peld/14s/giant-killer-chop-down?utm_source=api', 'layout': 'adventure', 'highres_image': True, 'image_uris': {'small': 'https://img.scryfall.com/cards/small/front/f/9/f999067b-f7af-4482-9549-108d21be7acc.jpg?1571851296', 'normal': 'https://img.scryfall.com/cards/normal/front/f/9/f999067b-f7af-4482-9549-108d21be7acc.jpg?1571851296', 'large': 'https://img.scryfall.com/cards/large/front/f/9/f999067b-f7af-4482-9549-108d21be7acc.jpg?1571851296', 'png': 'https://img.scryfall.com/cards/png/front/f/9/f999067b-f7af-4482-9549-108d21be7acc.png?1571851296', 'art_crop': 'https://img.scryfall.com/cards/art_crop/front/f/9/f999067b-f7af-4482-9549-108d21be7acc.jpg?1571851296', 'border_crop': 'https://img.scryfall.com/cards/border_crop/front/f/9/f999067b-f7af-4482-9549-108d21be7acc.jpg?1571851296'}, 'mana_cost': '{W} // {2}{W}', 'cmc': 1.0, 'type_line': 'Creature — Human Peasant // Instant — Adventure', 'power': '1', 'toughness': '2', 'colors': ['W'], 'color_identity': ['W'], 'card_faces': [{'object': 'card_face', 'name': 'Giant Killer', 'mana_cost': '{W}', 'type_line': 'Creature — Human Peasant', 'oracle_text': '{1}{W}, {T}: Tap target creature.', 'power': '1', 'toughness': '2', 'flavor_text': '"What size advantage? They never even see me coming."', 'artist': 'Jesper Ejsing', 'artist_id': 'a5f8354a-8b51-4e59-96b2-0e3aeae4fa1d', 'illustration_id': '4f255291-7e3c-4f3a-8c08-51b72e576696'}, {'object': 'card_face', 'name': 'Chop Down', 'mana_cost': '{2}{W}', 'type_line': 'Instant — Adventure', 'oracle_text': 'Destroy target creature with power 4 or greater. (Then exile this card. You may cast the creature later from exile.)', 'artist': 'Jesper Ejsing', 'artist_id': 'a5f8354a-8b51-4e59-96b2-0e3aeae4fa1d'}], 'legalities': {'standard': 'legal', 'future': 'legal', 'historic': 'legal', 'pioneer': 'legal', 'modern': 'legal', 'legacy': 'legal', 'pauper': 'not_legal', 'vintage': 'legal', 'penny': 'legal', 'commander': 'legal', 'brawl': 'legal', 'duel': 'legal', 'oldschool': 'not_legal'}, 'games': ['paper'], 'reserved': False, 'foil': True, 'nonfoil': False, 'oversized': False, 'promo': True, 'reprint': True, 'variation': False, 'set': 'peld', 'set_name': 'Throne of Eldraine Promos', 'set_type': 'promo', 'set_uri': 'https://api.scryfall.com/sets/5f593578-5079-4988-8e88-a3df186bdc2b', 'set_search_uri': 'https://api.scryfall.com/cards/search?order=set&q=e%3Apeld&unique=prints', 'scryfall_set_uri': 'https://scryfall.com/sets/peld?utm_source=api', 'rulings_uri': 'https://api.scryfall.com/cards/f999067b-f7af-4482-9549-108d21be7acc/rulings', 'prints_search_uri': 'https://api.scryfall.com/cards/search?order=released&q=oracleid%3A91e71279-c9d2-4873-916a-59da03a65741&unique=prints', 'collector_number': '14s', 'digital': False, 'rarity': 'rare', 'flavor_text': '"What size advantage? They never even see me coming."', 'card_back_id': '0aeebaf5-8c7d-4636-9e82-8c27447861f7', 'artist': 'Jesper Ejsing', 'artist_ids': ['a5f8354a-8b51-4e59-96b2-0e3aeae4fa1d'], 'illustration_id': '4f255291-7e3c-4f3a-8c08-51b72e576696', 'border_color': 'black', 'frame': '2015', 'full_art': False, 'textless': False, 'booster': False, 'story_spotlight': False, 'promo_types': ['prerelease', 'datestamped'], 'edhrec_rank': 12643, 'related_uris': {'tcgplayer_decks': 'https://decks.tcgplayer.com/magic/deck/search?contains=Giant+Killer&page=1&partner=Scryfall&utm_campaign=affiliate&utm_medium=scryfall&utm_source=scryfall', 'edhrec': 'https://edhrec.com/route/?cc=Giant+Killer', 'mtgtop8': 'https://mtgtop8.com/search?MD_check=1&SB_check=1&cards=Giant+Killer'}}
    delver_of_secrets: CardDescription = {'object': 'card', 'id': '28059d09-2c7d-4c61-af55-8942107a7c1f', 'oracle_id': 'edd531b9-f615-4399-8c8c-1c5e18c4acbf', 'multiverse_ids': [439326, 439327], 'mtgo_id': 66417, 'tcgplayer_id': 151638, 'name': 'Delver of Secrets // Insectile Aberration', 'lang': 'en', 'released_at': '2017-11-24', 'uri': 'https://api.scryfall.com/cards/28059d09-2c7d-4c61-af55-8942107a7c1f', 'scryfall_uri': 'https://scryfall.com/card/v17/7/delver-of-secrets-insectile-aberration?utm_source=api', 'layout': 'transform', 'highres_image': True, 'cmc': 1.0, 'type_line': 'Creature — Human Wizard // Creature — Human Insect', 'color_identity': ['U'], 'card_faces': [{'object': 'card_face', 'name': 'Delver of Secrets', 'mana_cost': '{U}', 'type_line': 'Creature — Human Wizard', 'oracle_text': 'At the beginning of your upkeep, look at the top card of your library. You may reveal that card. If an instant or sorcery card is revealed this way, transform Delver of Secrets.', 'colors': ['U'], 'power': '1', 'toughness': '1', 'artist': 'Lius Lasahido', 'artist_id': '0a0e9093-ce44-4a69-93a7-09b63e7c330d', 'illustration_id': 'f057ade6-99c3-4fd6-8818-a00bc0b28696', 'image_uris': {'small': 'https://img.scryfall.com/cards/small/front/2/8/28059d09-2c7d-4c61-af55-8942107a7c1f.jpg?1562701962', 'normal': 'https://img.scryfall.com/cards/normal/front/2/8/28059d09-2c7d-4c61-af55-8942107a7c1f.jpg?1562701962', 'large': 'https://img.scryfall.com/cards/large/front/2/8/28059d09-2c7d-4c61-af55-8942107a7c1f.jpg?1562701962', 'png': 'https://img.scryfall.com/cards/png/front/2/8/28059d09-2c7d-4c61-af55-8942107a7c1f.png?1562701962', 'art_crop': 'https://img.scryfall.com/cards/art_crop/front/2/8/28059d09-2c7d-4c61-af55-8942107a7c1f.jpg?1562701962', 'border_crop': 'https://img.scryfall.com/cards/border_crop/front/2/8/28059d09-2c7d-4c61-af55-8942107a7c1f.jpg?1562701962'}}, {'object': 'card_face', 'name': 'Insectile Aberration', 'mana_cost': '', 'type_line': 'Creature — Human Insect', 'oracle_text': 'Flying', 'colors': ['U'], 'color_indicator': ['U'], 'power': '3', 'toughness': '2', 'flavor_text': '"Unfortunately, all my test animals have died or escaped, so I shall be the final subject. I feel no fear. This is a momentous night."\n—Laboratory notes, final entry', 'artist': 'Lius Lasahido', 'artist_id': '0a0e9093-ce44-4a69-93a7-09b63e7c330d', 'illustration_id': '7719db45-6774-4005-85ab-ccc6923a1545', 'image_uris': {'small': 'https://img.scryfall.com/cards/small/back/2/8/28059d09-2c7d-4c61-af55-8942107a7c1f.jpg?1562701962', 'normal': 'https://img.scryfall.com/cards/normal/back/2/8/28059d09-2c7d-4c61-af55-8942107a7c1f.jpg?1562701962', 'large': 'https://img.scryfall.com/cards/large/back/2/8/28059d09-2c7d-4c61-af55-8942107a7c1f.jpg?1562701962', 'png': 'https://img.scryfall.com/cards/png/back/2/8/28059d09-2c7d-4c61-af55-8942107a7c1f.png?1562701962', 'art_crop': 'https://img.scryfall.com/cards/art_crop/back/2/8/28059d09-2c7d-4c61-af55-8942107a7c1f.jpg?1562701962', 'border_crop': 'https://img.scryfall.com/cards/border_crop/back/2/8/28059d09-2c7d-4c61-af55-8942107a7c1f.jpg?1562701962'}}], 'legalities': {'standard': 'not_legal', 'future': 'not_legal', 'historic': 'not_legal', 'pioneer': 'not_legal', 'modern': 'legal', 'legacy': 'legal', 'pauper': 'legal', 'vintage': 'legal', 'penny': 'not_legal', 'commander': 'legal', 'brawl': 'not_legal', 'duel': 'legal', 'oldschool': 'not_legal'}, 'games': ['mtgo', 'paper'], 'reserved': False, 'foil': True, 'nonfoil': False, 'oversized': False, 'promo': False, 'reprint': True, 'variation': False, 'set': 'v17', 'set_name': 'From the Vault: Transform', 'set_type': 'from_the_vault', 'set_uri': 'https://api.scryfall.com/sets/63c89a12-d115-4084-a4af-fceef40ca02f', 'set_search_uri': 'https://api.scryfall.com/cards/search?order=set&q=e%3Av17&unique=prints', 'scryfall_set_uri': 'https://scryfall.com/sets/v17?utm_source=api', 'rulings_uri': 'https://api.scryfall.com/cards/28059d09-2c7d-4c61-af55-8942107a7c1f/rulings', 'prints_search_uri': 'https://api.scryfall.com/cards/search?order=released&q=oracleid%3Aedd531b9-f615-4399-8c8c-1c5e18c4acbf&unique=prints', 'collector_number': '7', 'digital': False, 'rarity': 'mythic', 'card_back_id': '0aeebaf5-8c7d-4636-9e82-8c27447861f7', 'artist': 'Lius Lasahido', 'artist_ids': ['0a0e9093-ce44-4a69-93a7-09b63e7c330d'], 'border_color': 'black', 'frame': '2015', 'frame_effects': ['sunmoondfc'], 'full_art': False, 'textless': False, 'booster': False, 'story_spotlight': False, 'edhrec_rank': 6938, 'related_uris': {'gatherer': 'https://gatherer.wizards.com/Pages/Card/Details.aspx?multiverseid=439326', 'tcgplayer_decks': 'https://decks.tcgplayer.com/magic/deck/search?contains=Delver+of+Secrets&page=1&partner=Scryfall&utm_campaign=affiliate&utm_medium=scryfall&utm_source=scryfall', 'edhrec': 'https://edhrec.com/route/?cc=Delver+of+Secrets', 'mtgtop8': 'https://mtgtop8.com/search?MD_check=1&SB_check=1&cards=Delver+of+Secrets'}}
    life_from_the_loam: CardDescription = {'object': 'card', 'id': '7ac16d09-8bc7-407c-a757-666f4707bc90', 'oracle_id': 'ac8fba34-512c-4f24-999a-ab72f1ce4acb', 'multiverse_ids': [89001], 'mtgo_id': 23376, 'mtgo_foil_id': 23377, 'tcgplayer_id': 13340, 'name': 'Life from the Loam', 'lang': 'en', 'released_at': '2005-10-07', 'uri': 'https://api.scryfall.com/cards/7ac16d09-8bc7-407c-a757-666f4707bc90', 'scryfall_uri': 'https://scryfall.com/card/rav/172/life-from-the-loam?utm_source=api', 'layout': 'normal', 'highres_image': True, 'image_uris': {'small': 'https://img.scryfall.com/cards/small/front/7/a/7ac16d09-8bc7-407c-a757-666f4707bc90.jpg?1562109415', 'normal': 'https://img.scryfall.com/cards/normal/front/7/a/7ac16d09-8bc7-407c-a757-666f4707bc90.jpg?1562109415', 'large': 'https://img.scryfall.com/cards/large/front/7/a/7ac16d09-8bc7-407c-a757-666f4707bc90.jpg?1562109415', 'png': 'https://img.scryfall.com/cards/png/front/7/a/7ac16d09-8bc7-407c-a757-666f4707bc90.png?1562109415', 'art_crop': 'https://img.scryfall.com/cards/art_crop/front/7/a/7ac16d09-8bc7-407c-a757-666f4707bc90.jpg?1562109415', 'border_crop': 'https://img.scryfall.com/cards/border_crop/front/7/a/7ac16d09-8bc7-407c-a757-666f4707bc90.jpg?1562109415'}, 'mana_cost': '{1}{G}', 'cmc': 2.0, 'type_line': 'Sorcery', 'oracle_text': 'Return up to three target land cards from your graveyard to your hand.\nDredge 3 (If you would draw a card, instead you may put exactly three cards from the top of your library into your graveyard. If you do, return this card from your graveyard to your hand. Otherwise, draw a card.)', 'colors': ['G'], 'color_identity': ['G'], 'legalities': {'standard': 'not_legal', 'future': 'not_legal', 'historic': 'not_legal', 'pioneer': 'not_legal', 'modern': 'legal', 'legacy': 'legal', 'pauper': 'not_legal', 'vintage': 'legal', 'penny': 'not_legal', 'commander': 'legal', 'brawl': 'not_legal', 'duel': 'legal', 'oldschool': 'not_legal'}, 'games': ['mtgo', 'paper'], 'reserved': False, 'foil': True, 'nonfoil': True, 'oversized': False, 'promo': False, 'reprint': False, 'variation': False, 'set': 'rav', 'set_name': 'Ravnica: City of Guilds', 'set_type': 'expansion', 'set_uri': 'https://api.scryfall.com/sets/15fccbe8-2825-41ca-9d0a-67aebdf91c4a', 'set_search_uri': 'https://api.scryfall.com/cards/search?order=set&q=e%3Arav&unique=prints', 'scryfall_set_uri': 'https://scryfall.com/sets/rav?utm_source=api', 'rulings_uri': 'https://api.scryfall.com/cards/7ac16d09-8bc7-407c-a757-666f4707bc90/rulings', 'prints_search_uri': 'https://api.scryfall.com/cards/search?order=released&q=oracleid%3Aac8fba34-512c-4f24-999a-ab72f1ce4acb&unique=prints', 'collector_number': '172', 'digital': False, 'rarity': 'rare', 'watermark': 'golgari', 'card_back_id': '0aeebaf5-8c7d-4636-9e82-8c27447861f7', 'artist': 'Terese Nielsen', 'artist_ids': ['eb55171c-2342-45f4-a503-2d5a75baf752'], 'illustration_id': '8c957ece-09ad-4729-9898-227f0173ca6f', 'border_color': 'black', 'frame': '2003', 'full_art': False, 'textless': False, 'booster': True, 'story_spotlight': False, 'edhrec_rank': 409, 'related_uris': {'gatherer': 'https://gatherer.wizards.com/Pages/Card/Details.aspx?multiverseid=89001', 'tcgplayer_decks': 'https://decks.tcgplayer.com/magic/deck/search?contains=Life+from+the+Loam&page=1&partner=Scryfall&utm_campaign=affiliate&utm_medium=scryfall&utm_source=scryfall', 'edhrec': 'https://edhrec.com/route/?cc=Life+from+the+Loam', 'mtgtop8': 'https://mtgtop8.com/search?MD_check=1&SB_check=1&cards=Life+from+the+Loam'}}
    assert multiverse.name_from_card_description(fire_ice) == 'Fire // Ice'
    assert multiverse.name_from_card_description(hanweir_garrison) == 'Hanweir Garrison'
    assert multiverse.name_from_card_description(giant_killer) == 'Giant Killer'
    assert multiverse.name_from_card_description(delver_of_secrets) == 'Delver of Secrets'
    assert multiverse.name_from_card_description(life_from_the_loam) == 'Life from the Loam'
